import { Color, Style, Reset, wrap } from './ansi.js'
import { exec, partition } from './fn.js'

const { Yellow, Green, Red } = Color.Foreground
const { Bold } = Style

const reset = [Reset.All]
const style = (s, ...args) => wrap(args, reset)(s)
const fail = wrap([Red, Bold], reset)

const process = ([name, tests]) => {
    const results = tests.map(exec)

    const [passed, failed] = partition(test => test.success, results)

    return {
        name,
        passed,
        failed,
        results,
        toString: () => {
            const len = results.length
            const passLen = passed.length
            const failLen = failed.length

            let color = Yellow

            if (passLen === len) {
                color = Green
            } else if (failLen === len) {
                color = Red
            }
            const header = `${style(name, Bold, color)} (${passLen}/${len})`

            const text = results.map(result => {
                const status = result.success ? 'pass' : fail('fail')
                return `  ${result.description}... ${status}`
            }).join('\n')

            return `${header}\n${text}\n`
        }
    }
}

const makeSummary = results => {
    const [passed, failed] = results.reduce(
        ([pass, fail], result) => ([
            pass + result.passed.length,
            fail + result.failed.length,
        ]),
        [0, 0]
    )
    return {
        passed,
        failed,
        toString() {
            const failures = results
                .filter(result => result.failed.length)
                .flatMap(result => {
                    return result.failed.map(({ description, error }) => {
                        return `  ${result.name}::${description}\n${error.stack}`
                    }).join('\n\n')
                }).join('\n\n')

            const summary = `test result: ${failed}. ${passed} passed; ${failed} failed`

            return `failures:\n${failures}\n\n${summary}`
        }
    }
}
export const term = tests => {
    const suites = Object.entries(tests).map(process)
    suites.forEach(s => console.log(s.toString()))
    console.log(makeSummary(suites).toString())

}
